<?php

//  Copyright (C) 2002-2006 Balint Kis (balint@k-i-s.net)
//  Copyright (C) 2005-2007 Solomon Peachy (pizza@shaftnet.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

function validate_exif_string($exif_string) {
  $string_length = strlen($exif_string);
  $newstring = "";
  $j = 0;

  /* Remove non displayable ASCII characters */
  for ($i=0; $i < $string_length; $i++) {
    $ascii_num = ord($exif_string[$i]);

    if ($ascii_num == 0x27)
      continue;

    if ((0xff > $ascii_num) && ($ascii_num > 0x1F)) {
      $newstring = $newstring . $exif_string[$i];
      $j++;
    }
  }

  return pg_escape_string($newstring);
}

function replace_resolution_unit($string, $resolution_unit) {
  $trans = array("ResolutionUnit" => $resolution_unit);
  return strtr($string, $trans);
}

function convert_exif2sql_timestamp($timestamp) {
  if (!$timestamp)
    return "";

  $year   = substr($timestamp,  0, 4);
  $month  = substr($timestamp,  5, 2);
  $day    = substr($timestamp,  8, 2);
  $hour   = substr($timestamp, 11, 2);
  $minute = substr($timestamp, 14, 2);
  $second = substr($timestamp, 17, 2);

  if ($year == 0 || $month == 0 || $day == 0 || $hour == "" || $minute == "" || $second == "") {
    $today = getdate();
    return sprintf("%04d-%02d-%02d %02d:%02d:%02d", $today['year'], $today['mon'], $today['mday'], $today['hours'], $today['minutes'], $today['seconds']);
  }

  $sql_timestamp = $year.'-'.$month.'-'.$day.' '.$hour.':'.$minute.':'.$second;

  return $sql_timestamp;
}

/* Convert EXIF ratios (eg 60/10, 640/32) into a real number */
function convert_exif_ratio($ratio) {
  $pos = strpos($ratio, "/");
  if ($pos != false) {
    $value = substr($ratio, 0, $pos);
    $value /= substr($ratio, $pos + 1);
    $ratio = $value;
  }
  return $ratio;
}

function convert_exif_aperture($database, $aperture) {
  $aperture_id = "null";
  $minimum_distance = 10000;

  $all_apertures = pg_query($database, "select identifier, value from aperture");
  for ($item = 0; $item < pg_num_rows($all_apertures); $item++) {
    $current = pg_fetch_row($all_apertures, $item);
    $distance = abs($current[1] - $aperture);
    if ($minimum_distance > $distance) {
      $minimum_distance = $distance;
      $aperture_id = $current[0];
    }
    if ($minimum_distance == 0)
      break;
  }
  return $aperture_id;
}

function convert_exif_shutter($database, $shutter) {
  $pos = strpos($shutter, "/");
  if ($pos != false) {
    $effective_shutter = substr($shutter, 0, $pos);
    $effective_shutter /= substr($shutter, $pos+1);
  } else {
    $effective_shutter = $shutter;
  }

  $shutter_id = "null";
  $minimum_distance = 10000;

  $all_speeds = pg_query($database, "select identifier, value from shutter");
  for ($item = 0; $item < pg_num_rows($all_speeds); $item++) {
    $current = pg_fetch_row($all_speeds, $item);

    $pos = strpos($current[1], "/");
    if ($pos === false) {
      $distance = abs($current[1] - $effective_shutter);
    }
    else {
      $effective_tmp_shutter = substr($current[1], 0, $pos);
      $effective_tmp_shutter /= substr($current[1], $pos + 1);
      $distance = abs($effective_shutter - $effective_tmp_shutter);
    }

    if ($minimum_distance > $distance) {
      $minimum_distance = $distance;
      $shutter_id = $current[0];
    }

    if ($minimum_distance == 0)
      break;
  }
  return $shutter_id;
}

function convert_exif_ev_comp($database, $ev_comp) {
  $ev_comp_id = "null";
  $minimum_distance = 10000;

  $all_ev_comps = pg_query($database, "select identifier, value from ev_comp");
  for ($item = 0; $item < pg_num_rows($all_ev_comps); $item++) {
    $current = pg_fetch_row($all_ev_comps, $item);
    $distance = abs($current[1] - $ev_comp);
    if ($minimum_distance > $distance) {
      $minimum_distance = $distance;
      $ev_comp_id = $current[0];
    }
    if ($minimum_distance == 0)
      break;
  }

  return $ev_comp_id;
}

function convert_exif_focal_length($database, $focal_length) {
  $focal_length_id = "null";
  $minimum_distance = 10000;

  $all_focal_lengths = pg_query($database, "select identifier, value from focal_length");
  for ($item = 0; $item < pg_num_rows($all_focal_lengths); $item++) {
    $current = pg_fetch_row($all_focal_lengths, $item);
    $distance = abs($current[1] - $focal_length);
    if ($minimum_distance > $distance) {
      $minimum_distance = $distance;
      $focal_length_id = $current[0];
    }
    if ($minimum_distance == 0)
      break;
  }
  return $focal_length_id;
}

function convert_exif_iso_film($database, $user_id, $iso) {
  $all_digital_films = pg_query($database, "
     select film.identifier, film_iso.value
       from film, film_type, film_format, film_iso
      where film.users = $user_id
        and film.type = film_type.identifier
        and film_type.format = film_format.identifier
        and film_format.value = 'Digital'
        and film_type.iso = film_iso.identifier");

  /* unlike elsewhere w look for exact matches */
  for ($item = 0; $item < pg_num_rows($all_digital_films); $item++) {
    $current = pg_fetch_row($all_digital_films, $item);
    if ($current[1] == $iso)
      return $current[0];
  }

  return "null";
}

function convert_exif_camera($database, $user_id, &$image_data, $camera_make, $camera_model) {
  $camera = pg_query($database, "
     select camera.identifier, camera_type.raw_icc_profile, camera.ignore_comment
       from camera, camera_type, manufacturer
      where '$camera_make' ~*  manufacturer.name
        and '$camera_model' ~*  camera_type.model
        and camera_type.identifier = camera.type
        and camera_type.manufacturer = manufacturer.identifier
        and camera.users = $user_id");

  if (!$camera)
    return FALSE;

  if (pg_num_rows($camera) > 0) {
    $this_camera = pg_fetch_row($camera, 0);
    $image_data['camera'] = $this_camera[0];
    $image_data['camera_input_profile'] = $this_camera[1];
    if ($this_camera[2])
      $image_data['IgnoreExifUserComment'] = TRUE;

    return TRUE;
  }

  return FALSE;
}

function convert_exif_lens($database, $user_id, &$image_data, $lens_id) {
  $lens_id = pg_escape_string($lens_id);

  $lens = pg_query($database, "select lens.identifier 
                                 from lens_type, lens
                                where lens_type.identifier = lens.type
                                  and exiftool_lens_id = '$lens_id' 
                                  and lens.users = $user_id");

  if (!$lens)
    return FALSE;

  if (pg_num_rows($lens) > 0) {
    $my_lens = pg_fetch_row($lens, 0);
    $image_data['lens'] = $my_lens[0];
    return TRUE;
  }

  return FALSE;
}

function exif_string_helper(&$image_data, $exif_data, $image_key, $exif_key) {
  $tmp = pg_escape_string($exif_data[$exif_key]);
  if (strlen($tmp)) {
    $image_data[$image_key] = $tmp;
  } 
}

function photo_parse_exif_orientation_colorspace($database, $index, $user_id, $image_data, $tag) {
  $exif_data = $image_data["exiftool_data_$tag"];

  /* Orientation -- Map the new to the old */
  switch ($exif_data["Orientation"]) {
  case 'Mirror horizontal':  /* 2 */
    $image_data["orientation"] = "Mirrored";
    break;
  case 'Rotate 180':  /* 3 */
    $image_data["orientation"] = "Upsidedown";
    break;
  case 'Mirror vertical':  /* 4 */
    $image_data["orientation"] = "Upsidedown Mirrored";
    break;
  case 'Mirror horizontal and rotate 270 CW': /* 5 */
    $image_data["orientation"] = "90 deg CW Mirrored";
    break;
  case 'Rotate 90 CW': /* 6 */
    $image_data["orientation"] = "90 deg CW";
    break;
  case 'Mirror horizontal and rotate 90 CW': /* 7 */
    $image_data["orientation"] = "90 deg CCW Mirrored";
    break;
  case 'Rotate 270 CW': /* 8 */
    $image_data["orientation"] = "90 deg CCW";
    break;
  case 'Horizontal (normal)': /* 1 */
    $image_data["orientation"] = "Normal (0 deg)";
    break;
  }

  $image_data["orientation_orig"] = $image_data["orientation"];

  /* So are Color Spaces */
  if (isset($exif_data["ICC Profile"])) 
    $image_data["colorspace"] = 99; /* Embedded Profile */
  else if (isset($exif_data["Profile Class"]))
    $image_data["colorspace"] = 99; /* Embedded Profile */
  else if (substr_compare($image_data['file'][$index]['original_name'], "_", 0, 1) === 0)
    $image_data["colorspace"] = 2; /* Adobe RGB */
  else if (isset($exif_data["Color Space"])) {
    $image_data["colorspace"] = 1; 	/* Default to sRGB */
    if ($exif_data["Color Space"] == "Adobe RGB")
      $image_data["colorspace"] = 2; /* Adobe RGB */
    else if ($exif_data["Color Space"] != "sRGB")
      $image_data["colorspace"] = 1; /* Anything other than 'sRGB' is undefined, but we have to use it anway... */
  }

  if (isset($exif_data["Camera Model Name"]) ||
      isset($exif_data["Make"])) {
    convert_exif_camera($database, $user_id, $image_data, validate_exif_string($exif_data["Make"]), validate_exif_string($exif_data["Camera Model Name"]));
  }

  return $image_data;
}

function photo_parse_exif($database, $index, $user_id, $image_data) {
  global $strings;

  $image_data = extract_exiftool_metadata($image_data, $index, "EXIF");

  $image_data["exif"] = "";
  $exif_data = $image_data['exiftool_data_EXIF'];

  /* Process the raw data */
  foreach ($exif_data as $key => $value) {
    $image_data["exif"] .= "<key>$key</key><value>$value</value>";

    switch ($key) {
    case "Artist":
      exif_string_helper($image_data, $exif_data, "author", $key);
      break;
    case "Copyright":
      exif_string_helper($image_data, $exif_data, "copyright", $key);
      break;
    case "Date/Time Original":
      $image_data["date_of_exposure"] = convert_exif2sql_timestamp($value);
      break;
    case "Create Date":
      if ($image_data["date_of_exposure"] == "")
	$image_data["date_of_exposure"] = convert_exif2sql_timestamp($value);
      break;
    case "Camera Model Name":
      convert_exif_camera($database, $user_id, $image_data, validate_exif_string($exif_data["Make"]), validate_exif_string($exif_data["Camera Model Name"]));
      break;
    case "ISO":
      $image_data["film"] = convert_exif_iso_film($database, $user_id, $value);
      break;
    case "Flash Exposure Compensation":
      $image_data["flash_comp"] = convert_exif_ev_comp($database, $value);
      break;
    case "Exposure Compensation":
      $image_data["exp_comp"] = convert_exif_ev_comp($database, $value);
      break;
    case "Exposure Difference":
      $image_data["exp_diff"] = convert_exif_ev_comp($database, $value);
      break;
    case "F Number":
      $image_data["aperture"] = convert_exif_aperture($database, $value);
      break;
    case "Shutter Speed":
      $image_data["shutter"] = convert_exif_shutter($database, $value); 
      break;
    case "Lens ID":
      convert_exif_lens($database, $user_id, $image_data, $value);
      break;
    case "GPS Latitude":
      $image_data['latitude'] = parse_latitude($value);
      break;
    case "GPS Longitude":
      $image_data['longitude'] = parse_latitude($value);
      break;
    case "GPS Altitude":
      $regexp = "/((?=\d|\.\d)\d*(?:\.\d*)?)/";
      $res = array();
      
      if (preg_match($regexp, $value, $res)) {
	$image_data['altitude'] = $res[1];
	if ($exif_data['GPS Altitude Ref'] != 'Above Sea Level') 
	  $image_data['altitude'] = - $image_data['altitude'];
      }
      break;
    case "GPS Img Direction":
      $image_data['direction'] = $value;
      break;
    case "Jpg From Raw":
      $image_data['jpgfromraw'] = TRUE;
      break;
    }
  } 
    
  /* Focal length is special */
  $tmp = $exif_data["Focal Length"];
  $tmp = substr($tmp, 0, strpos($tmp, 'mm'));

  /* Image Comment */
  if (isset($exif_data["User Comment"]) && !$image_data['IgnoreExifUserComment']) {
    exif_string_helper($image_data, $exif_data, "title", "User Comment");
  }

  if ($exif_data["Scale Factor To 35mm Equivalent"] > 2) {
    if ($exif_data["Focal Length In 35mm Format"]) {
      $tmp = $exif_data["Focal Length In 35mm Format"];
      $tmp = substr($tmp, 0, strpos($tmp, 'mm'));
    } else {
      $tmp = $tmp * $exif_data["Scale Factor To 35mm Equivalent"];
    }
  }
    
  if (isset($exif_data["Focal Length"]))
    $image_data["focal_length"] = convert_exif_focal_length($database, $tmp);

  /* Pull in orientation and colorspace data */
  $image_data = photo_parse_exif_orientation_colorspace($database, $index, $user_id, $image_data, "EXIF");
    
  // New:  "metering" "program" "flash" and "flash mode" 
  
  /* Escape it for database sanity */
  $image_data["exif"] = pg_escape_string($image_data["exif"]);

  return $image_data;
}

?>
