<?php

//  Copyright (C) 2002-2006 Balint Kis (balint@k-i-s.net)
//  Copyright (C) 2005-2007 Solomon Peachy (pizza@shaftnet.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

include_once "common.php";
include_once "iptc.php";
include_once "exif.php";

function convert_rdf2sql_timestamp($date, $time) {
  $year   = substr($date,  0, 4);
  $month  = substr($date,  5, 2);
  $day    = substr($date,  8, 2);

  if ($date == "" || $year == 0 || $month == 0 || $day == 0) {
    $today = getdate();
    $year   = $today['year'];
    $month  = $today['mon'];
    $day    = $today['mday'];
  }

  $hour   = substr($time,  0, 2);
  $minute = substr($time,  3, 2);
  $second = substr($time,  6, 2);

  if ($date == "") {
    $hour   = "00";
    $minute = "00";
    $second = "00";
  }

  $sql_timestamp = $year.'-'.$month.'-'.$day.' '.$hour.':'.$minute.':'.$second;
  return $sql_timestamp;
}

function photo_parse_rdf($database, $index, $user_id, $image_data) {
  global $rdf_image_data;
  global $metering_mode_conversion;
  global $exposure_program_conversion;
  global $strings;

  $loc = "";
  $city = "";
  $state = "";
  $country = "";
  $camera_make = "";
  $camera_model = "";
  $keywords = "";
  
  $image_data = extract_exiftool_metadata($image_data, $index, "XMP");
  /* Populate RDF data */
  $rdf_data = $image_data['exiftool_data_XMP'];
  foreach ($rdf_data as $key => $value) {
    $image_data["rdf"] .= "<key>$key</key><value>$value</value>";
    switch ($key) {
    case "Creator": // and 'Owner' too
      $image_data["author"] = $value;
      break;
    case "Rights": // and 'Usage Terms' too?
      $image_data["copyright"] = $value;
      break;    
    case "Headline":
      $image_data["headline"] = $value;
      break;    
    case "Title":
      $image_data["title"] = $value;
      break;    
    case "Description":  // this could go into caption..?
      $image_data["remark"] = $value;
      break;    
    case "Date Created": // and 'Create Date' and 'Date'
      $image_data["date_of_exposure"] = convert_rdf2sql_timestamp($value, "00:00:00");
      break;
    case "City":
      $city = $value;
      break;
    case "State":
      $state = $value;
      break;
    case "Country":
      $country = $value;
      break;
    case "Location":
      $loc = $value;
      break;
    case "Instructions":  
      $image_data["instructions"] = $value;
      break;    
    case "Caption Writer":  
      $image_data["caption_writer"] = $value;
      break;    
    case "Web Statement":  
      $image_data["web_statement"] = $value;
      break;    
    case "Source":  
      $image_data["source"] = $value;
      break;    
    case "Credit":  
      $image_data["credit"] = $value;
      break;    
    case "Category":  
      $image_data["category"] = $value;
      break;    
    case "Transmission Reference":  
      $image_data["transmission_reference"] = $value;
      break;    
    case "Supplemental Categories":   // should get shoved onto keywords?
      $image_data["supplemental_category"] = $value;
      break;    
    case "Make":
      $camera_make = $value;
      break;
    case "Model":
      $camera_model = $value;
      break;
    case "Subject":
      $keywords = $value;
      break;
    default:
      print "<li>UNHANDLED: $key = $value</li>";
      break;
    }
  }

  if (($loc != "") || ($city != "") || ($state != "") || ($country != "")) {
    $image_data['location'] = 
      convert_iptc_location($database, $user_id,
                            $city, $state, $country);
  } else {
    $image_data['location'] = "0";
  }

  if (($camera_make != "") ||
      ($camera_model != "")) {
    $image_data['camera'] = convert_exif_camera($database, $user_id, $camera_make, $camera_model);
  }

  if ($keywords != "") {
    $keys = explode(", ", $keywords);
    foreach ($keys as $key) {
      $image_data['keywords'][] = $key;
    }
  }

  /* 

  $rdf_image_data["lens"]                   = "null";
  $rdf_image_data["film"]                   = "null";
  $rdf_image_data["flash"]                  = "null";
  $rdf_image_data["focal_length"]           = "null";
  $rdf_image_data["aperture"]               = "null";
  $rdf_image_data["shutter"]                = "null";
  $rdf_image_data["camera_metering"]        = "null";
  $rdf_image_data["camera_program"]         = "null";
  $rdf_image_data["flash_mode"]             = "null";

  */

  /* Escape it for database sanity */
  $image_data["rdf"] = pg_escape_string($image_data["rdf"]);

  return $image_data;
}

function convert_rdf_lens_serial_number($database, $user_id, $serial_number) {
  $lens = pg_fetch_row(pg_query($database, "
     select identifier from lens
      where serial_number=$serial_number
        and users=$user_id"));
  if ($lens)
    return $lens[0];

  return "null";
}

?>
