<?php

//  Copyright (C) 2002-2006 Balint Kis (balint@k-i-s.net)
//  Copyright (C) 2005-2007 Solomon Peachy (pizza@shaftnet.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

include_once "config.php";
include_once "common.php";
include_once "vars.php";
include_once "database.php";

function site_prolog($min_type = PO_USER_TYPE_DISABLED) {
  global $po_user;
  global $po_options;
  global $po_options_default;
  global $default_cred_timeout;
  global $po_session_lifetime;
  global $po_session_path;
  global $strings;
  global $theme;
  global $dayNames;
  global $dayNamesShort;
  global $monthNames;
  global $monthNamesShort;
  global $generate_i18n_data;

  global $passwords;

  $sybase = FALSE;
  // if magic quotes is enabled
  if ( get_magic_quotes_gpc () ) {
    // workaround because magic_quotes does not change $_SERVER['argv']
    $argv = isset($_SERVER['argv']) ? $_SERVER['argv'] : NULL; 
    
    // fix all affected arrays
    foreach ( array ('_ENV', '_REQUEST', '_GET', '_POST', '_COOKIE', '_SERVER') as $var ) {
      $GLOBALS[$var] = fix_magic_quotes ($GLOBALS[$var], $sybase);
    }
    $_SERVER['argv'] = $argv;
    
    // turn off magic quotes, this is so scripts which
    // are sensitive to the setting will work correctly
    ini_set ('magic_quotes_gpc', 0);
  }
  
  set_magic_quotes_runtime(0);
  
  /* Turn off error display */
  ini_set ('display_errors', 0);
  
  /* Establish DB Connection */
  $dbh = po_dbconnect();
  
  if ($dbh == FALSE) return FALSE;

  /* Start up session */
  ini_set("session.cache_expire", $po_session_lifetime / 60);
  ini_set("session.gc_maxlifetime", "$po_session_lifetime");
  if ($po_session_path != "") session_save_path($po_session_path);
  session_name("po_session");
  session_start();

  /* See if user is logged in; Set up session data.. */
  if (isset($_SESSION['po_user'])) {
    $new_user = $_SESSION['po_user'];

    foreach ($new_user as $key => $value) {
      $po_user[$key] = $new_user[$key];
    }

    /* Set last request time */
    $po_user['last_request'] = time();
  }

  $po_user['session_id'] = session_id();

  /* And then set up option data */
  if (isset($_SESSION['po_options'])) {
    $options = $_SESSION['po_options'];
    
    foreach ($options as $key => $value) {
      $po_options[$key] = $options[$key];
    }
  }

  /* Periodically refresh our cached credentials & preferences */
  if (($po_user['id'] != 0) && 
      ($po_user['last_updated'] + $default_cred_timeout) < time()) {
    /* Update preferences and credentials */
    refresh_user_prefs($dbh);
  }

  $tmp = array_merge($po_user['passwords'], $po_options['passwords']);
  $passwords = implode("," , $tmp);

  /* See if request updated things any -- Don't store! */
  if (isset($_REQUEST['order_by']) && isset($_REQUEST['order_by_req'])) {
    $po_options['order_by'] = $_REQUEST['order_by'];
  }
  if (isset($_REQUEST['photo_view'])) {
    $po_options['photo_view'] = $_REQUEST['photo_view'];
  }

  if (isset($_REQUEST['photos_per_page']) && isset($_REQUEST['per_page_req'])) {
    $po_options['photos_per_page'] = $_REQUEST['photos_per_page'];
  }
  if (isset($_REQUEST['rows_list_view']) && isset($_REQUEST['per_page_req'])) {
    $po_options['rows_list_view'] = $_REQUEST['rows_list_view'];
  }

  /* If a language tag is set, use it. */
  if (isset($_REQUEST['lang']))
    $po_options['lang'] = $_REQUEST['lang'];

  /* Update session details. */
  site_update_session(FALSE);

  /* And now, close it out to release locks */
  session_write_close();

  /* Sanity check in case the cookie gets screwed up */
  if (($min_type == PO_USER_TYPE_DISABLED) && !$po_user['type']) {
    session_start();
    setcookie(session_name(), '', time()-3600);
    session_destroy();    
  }
  
  /* Ensure user has minimum permissions for this page */
  if ($po_user['type'] < $min_type) {
    switch ($min_type) {
    case PO_USER_TYPE_ADMIN:
      site_push_error($strings['errors']['admin_perm']);
      break;
    case PO_USER_TYPE_USER:
      site_push_error($strings['errors']['user_perm']);
      break;
    case PO_USER_TYPE_CLIENT:
      site_push_error($strings['errors']['client_perm']);
      break;
    default:
      site_push_error($strings['errors']['insuff_perm']);
      break;
    }

    header("Location: login.php");
    site_epilog($dbh);
    exit();
  }

  /* Set up the locale settings and whatnot */
  include "lang/".$po_options['lang'].".php";

  foreach ($generate_i18n_data as $fn) {
	$fn();
  }

  setlocale(LC_ALL, $strings['formats']['locale_name']);
  
  return $dbh;
}

/* Updates the session state to match current configuration */
function site_update_session($start_sess=TRUE) {
  global $po_options;
  global $po_user;
  
  if ($po_user['session_id']) 
    session_id($po_user['session_id']);

  if ($start_sess) 
    session_start();

  if ($po_user['auto_login']) {
    $res = setcookie(session_name(), session_id(), time()+157680000);
    // site_push_error("auto_login = $po_user[auto_login], cookie = $res, sess = $po_user[session_id], start_sess = $start_sess");
    $po_user['auto_login'] = FALSE;
  }

  $_SESSION['po_options'] = $po_options;
  $_SESSION['po_user'] = $po_user;  

  if ($start_sess)
    session_write_close();
}

function site_epilog($database) {
  pg_close($database);
}

function site_adv() {
  global $ad_script;

  if ($ad_script) {
    print "<div class=\"banner_adv\">";
    print $ad_script;
    print "</div>";
  }
}

function site_logo() {
  global $thm_elem;

  print "<div id=\"site_logo\">$thm_elem[site_logo]</div>\n";

}

function site_credit() {
  global $strings;
  global $po_version;

  print "<div class=\"site_credit\">";

    print $strings['generic']['po_powered_by'] ." v$po_version<br/>\n";
    print $strings['generic']['po_copyright'];

    print "</div>\n";
}

function site_head($page_title, $title_b = "") {
  global $compress_pages;
  global $site_title;
  global $thm_elem;
  global $strings;

  if ($compress_pages &&
      ini_get('zlib.output_compression') == '' &&
      ini_get('zend_accelerator.compress_all') == '' &&
      ini_get('output_handler') != 'ob_gzhandler') {
    if (ob_get_level()) {
      ob_end_clean();
    }
    ob_start('ob_gzhandler');
  }

  header("Content-Type: text/html; charset=".$strings['formats']['encoding']);
/*
  header("Content-Type: application/xhtml+xml");
  print "<?xml version=\"1.0\" encoding=\"".$strings['formats']['encoding']."\"?>\n";
*/
  print "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">";
  print "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"".$strings['formats']['lang_xml']."\" lang=\"".$strings['formats']['lang_xml']."\" dir=\"".$strings['formats']['direction']."\">";
  print "<head>\n";
  print "<title>$site_title : $page_title". ($title_b != "" ? " : $title_b" : "") ."</title>\n";

  print "<!-- compliance patch for microsoft browsers -->\n";
  print "<!--[if lt IE 8]><script src=\"ie7/ie7-standard-p.js\" type=\"text/javascript\"></script><![endif]-->\n";
  print "<!--[if lt IE 8]><script src=\"ie7/ie7-css-strict.js\" type=\"text/javascript\"></script><![endif]-->\n";

  print "<meta http-equiv=\"Content-Type\" content=\"text/html;charset=".$strings['formats']['encoding']."\" />";

  print $thm_elem['shortcut_icon'];
  print $thm_elem['stylesheets'];

  print "<script type=\"text/javascript\" src=\"javascript/select.js\"></script>";
  print $thm_elem['javascript'];

  print "</head>\n";
}

function site_header($page_title, $title_b = "") {
  global $po_user;
  global $install_enabled;
  global $strings;

  site_head($page_title, $title_b);

  print "<body>\n";

  print "<script type=\"text/javascript\" src=\"javascript/wz_tooltip.js\"></script>\n";
  
  if (($po_user['type'] == PO_USER_TYPE_ADMIN) &&
      ($po_user['username'] == 'admin') &&
      ($po_user['password'] == 'admin')) {
    site_push_error($strings['errors']['default_admin']);
  }

  if ($install_enabled) {
    site_push_error($strings['errors']['install_enabled']);
  }

  site_dump_errors();

  site_logo();
  site_adv();

  print "<div id=\"content\">\n";
}

function site_foot() {
  global $strings;

  print "<script type=\"text/javascript\" src=\"javascript/jscalendar/calendar.js\"></script>";
  print "<script type=\"text/javascript\" src=\"javascript/jscalendar/lang/calendar-".$strings['formats']['lang_xml'].".js\"></script>";
  print "<script type=\"text/javascript\" src=\"javascript/jscalendar/calendar-setup.js\"></script>";

  print "</body></html>\n";
}

function err_str($error) {
  return "<font color=\"red\">$error</font>";
}

function site_push_error($error) {
  global $po_user;

  session_id($po_user['session_id']);

  session_start();
  $_SESSION['errors'][] = $error;
  session_write_close();
}

function site_dump_errors() {
  global $po_user;

  session_id($po_user['session_id']);
  session_start();

  if (isset($_SESSION['errors']) && count($_SESSION['errors']) > 0) {
    print "<div class=\"errorbar\">\n";
    print "<ul>\n";
    foreach ($_SESSION['errors'] as $error) {
      print "<li>$error</li>\n";
    }
    print "</ul>\n";
    print "</div>\n";
  }

  $_SESSION['errors'] = array();
  session_write_close();
}

function site_footer($database) {
  print "</div> <!-- id=content --> \n";

  site_credit();
  site_foot();
}

function site_navigator_status($left_string, $right_string) {
  print "<!-- Navigator Status Begin -->\n";
  print "<div class=\"status\">";
  print "<table><tr>";
  print "<td>";
  print "$left_string\n";
  print "</td><td>";
  print "$right_string\n";
  print "</td></tr></table>";
  print "</div>";
  print "<!-- Navigator Status End -->\n";
}

function site_navigator_stand_alone($database, $sub_menu) {
  site_navigator($sub_menu);
}

function site_search_entry($align, $size) {
  global $strings;
  global $thm_elem;

  print "<form action=\"search.text.general.php\" method=\"post\" accept-charset=\"".$strings['formats']['encoding']."\" >";
  print "<table align=\"center\" border=\"0\">";
  print "<tr><td  align=\"center\">";
  print "<input type=\"text\" name=\"search_string\" value=\"$search_string\" size=\"$size\" title=\"".$strings['search']['search_string_prompt']."/>";
  print "</td><td align=\"center\">";
  print $thm_elem['button.search'];
  print "</td></tr>";
  print "</table>";
  print "</form>";
}

function site_display_photo($database, $identifier, $framed, $version = FALSE) {
  global $access;
  global $po_user;
  global $po_options;
  global $passwords;

  if ($version === FALSE) {
    $version = "photo_version.master = 't'";
  } else {
    $version = "photo_version.identifier = '$version' ";
  }

  $query = 
    "select photo.identifier, photo.caption, photo.date_of_exposure, photo.date_added,
            photo.access_rights,
            11 - (select avg(rating.value) from rating where rating.photo=photo.identifier) as rating,
            views, photo.users, 
            photo.title, photo.author, photo_version.identifier as version
       from photo, photo_version
      where photo.identifier = '$identifier'
        and photo.identifier = photo_version.photo
        and $version
        and can_access_photo(photo.identifier, $po_user[id], '{".$passwords."}')";

  $photo_data = pg_query($database, $query);

  if (pg_num_rows($photo_data) < 1)
    return;

  display_photo_slides("folder", "", $photo_data, 0, 1, $framed, FALSE);
}

function site_display_random_photos($database, $type, $identifier, $num_of_photos, $framed) {
  global $po_user;
  global $access;
  global $po_options;
  global $passwords;

  switch ($type) {
  case "user":
    $sql_photo_selector = "photo.users = ".$identifier;
    $sql_extra_tables = "";
    $display_type = "folder";
    break;
  case "album":
    $sql_photo_selector = "photo_version.identifier = album_content.version and photo.identifier = album_content.photo and album_content.album=".$identifier;
    $sql_extra_tables = ", album_content";
    $display_type = "album";
    break;
  default:
  case "folder":
    $sql_photo_selector = "photo.folder = ".$identifier;
    $sql_extra_tables = "";
    $display_type = "folder";
    break;
  }

  $query = 
    "select photo.identifier, photo.caption, photo.date_of_exposure, photo.date_added, photo.access_rights,
            11 - (select avg(rating.value) from rating where rating.photo=photo.identifier) as rating,
            views, photo.users,
            photo.title, photo.author, photo_version.identifier as version
       from photo, photo_version $sql_extra_tables
      where $sql_photo_selector
        and photo.identifier = photo_version.photo
        and can_access_photo(photo.identifier, $po_user[id], '{".$passwords."}')
      order by random()
      limit $num_of_photos";

    $photo_data = pg_query($database, $query);

  $num_of_photos = pg_num_rows($photo_data);
  display_photo_slides($display_type, $identifier, $photo_data, 0, $num_of_photos, $framed, FALSE);
}

function site_display_user_name($database, $identifier, $link) {
  $user_name = pg_fetch_row(pg_query($database, "select first_name, last_name from users where identifier = $identifier"));

  if ($link)
    print "<a href=\"user.php?user=$identifier\">$user_name[0] $user_name[1]</a>";
  else
    print "$user_name[0] $user_name[1]";
}

?>
