--  Copyright (C) 2007 Solomon Peachy (pizza@shaftnet.org)

--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.

--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.

--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

----------------------------------------------------------
-- v2.33 to v2.34 patch
----------------------------------------------------------

-- version update
update version set identifier=3, description='2.34 customer release', major=2, minor=34, variation='A';

-- Seed date_added with our best guess:

update photo set date_added = 
 (select d.date_of_creation from photo_dupe d 
   where d.photo = photo.identifier) where date_added is NULL;

-- version update II
update version set identifier=3, description='2.34 customer release', 
 major=2, minor=34, variation='B';

-- Initialize last_modified timestamps;

update photo_version set date_changed = date_of_creation;

alter table photo_version alter column date_changed set not null;

update photo set date_changed = date_added;

alter table photo alter column date_changed set not null;

update folder set date_changed = 
     (select max(date_changed) from photo 
       where photo.folder = folder.identifier);

update folder set date_changed = 
     (select max(date_of_creation) from folder f2 
       where f2.parent_folder = folder.identifier) 
 where date_changed is null;

update folder set date_changed = date_of_creation 
 where date_changed is null;

update folder set date_changed = date_of_creation 
 where date_changed < date_of_creation;

alter table folder alter column date_changed set not null;

update album set date_changed =
  (select max(date_changed) from photo, album_content 
    where album_content.album = album.identifier 
      and album_content.photo = photo.identifier);

update album set date_changed =
     (select max(date_of_creation) from album f2
       where f2.parent_album = album.identifier)
 where date_changed is null;

update album set date_changed = date_of_creation 
 where date_changed is null;

update album set date_changed = date_of_creation 
 where date_changed < date_of_creation;

alter table album alter column date_changed set not null;

--- album_content timestamp..

update album_content set date_added =
 (select date_added from photo 
   where photo.identifier = album_content.photo);

alter table album_content alter column date_added set not null;

--- Auto-update triggers!

CREATE OR REPLACE FUNCTION album_content_date_added_trig() RETURNS "trigger" AS '
BEGIN

IF TG_OP = ''DELETE'' THEN
  UPDATE album SET date_changed = now()
   WHERE album.identifier = OLD.album;
  RETURN OLD;
END IF;

NEW.date_added = now();

UPDATE album SET date_changed = now()
 WHERE album.identifier = NEW.album;

RETURN NEW;

END
' LANGUAGE 'plpgsql' VOLATILE;


CREATE TRIGGER album_content_trigger
 BEFORE INSERT OR DELETE
    ON album_content
   FOR EACH ROW EXECUTE PROCEDURE album_content_date_added_trig();

CREATE OR REPLACE FUNCTION photo_date_changed_trig() RETURNS "trigger" AS '
BEGIN

IF TG_OP = ''DELETE'' THEN
  UPDATE folder SET date_changed = now() where identifier = OLD.folder;
  RETURN OLD;
END IF;

IF TG_OP = ''INSERT'' THEN
  UPDATE folder SET date_changed = now() where identifier = NEW.folder;
  NEW.date_changed = now();
  NEW.date_added = now();
  RETURN NEW;
END IF;

IF NEW.views = OLD.views THEN
  NEW.date_changed = now();
END IF;

IF NEW.folder != OLD.folder THEN
  UPDATE folder SET date_changed = now() where identifier = NEW.folder;
  UPDATE folder SET date_changed = now() where identifier = OLD.folder;
END IF;

RETURN NEW;

END;
' LANGUAGE 'plpgsql' VOLATILE;


CREATE TRIGGER photo_date_changed_trigger
 BEFORE INSERT OR UPDATE OR DELETE
    ON photo
   FOR EACH ROW EXECUTE PROCEDURE photo_date_changed_trig();

CREATE OR REPLACE FUNCTION photo_tech_date_changed_trig() RETURNS "trigger" AS '
BEGIN

IF TG_OP = ''DELETE'' THEN
 UPDATE photo SET date_changed = now() 
  WHERE photo.identifier = OLD.photo;
  RETURN OLD;
END IF;

UPDATE photo SET date_changed = now() 
 WHERE photo.identifier = NEW.photo;

RETURN NEW;

END;
' LANGUAGE 'plpgsql' VOLATILE;


CREATE TRIGGER photo_tech_date_changed_trigger
 BEFORE INSERT OR UPDATE
    ON photo_tech
   FOR EACH ROW EXECUTE PROCEDURE photo_tech_date_changed_trig();

CREATE TRIGGER photo_keywords_date_changed_trigger
 BEFORE INSERT OR UPDATE OR DELETE
    ON photo_keywords
   FOR EACH ROW EXECUTE PROCEDURE photo_tech_date_changed_trig();



CREATE OR REPLACE FUNCTION photo_version_date_changed_trig() RETURNS "trigger" AS '
BEGIN

IF TG_OP = ''DELETE'' THEN
  UPDATE photo SET date_changed = now() 
   WHERE photo.identifier = OLD.photo;

  RETURN OLD;
END IF;

IF TG_OP = ''INSERT'' THEN
   NEW.date_of_creation = now();
END IF;

UPDATE photo SET date_changed = now() 
 WHERE photo.identifier = NEW.photo;

NEW.date_changed = now();

RETURN NEW;

END;
' LANGUAGE 'plpgsql' VOLATILE;

CREATE TRIGGER photo_version_date_changed_trigger
 BEFORE INSERT OR UPDATE
    ON photo_version
   FOR EACH ROW EXECUTE PROCEDURE photo_version_date_changed_trig();



CREATE OR REPLACE FUNCTION date_changed_folder() RETURNS "trigger" AS '
BEGIN

IF TG_OP = ''DELETE'' THEN
  UPDATE folder SET date_changed = now() where identifier = OLD.parent_folder;
  RETURN OLD;
END IF;

IF TG_OP = ''INSERT'' THEN
  NEW.date_of_creation = now();
  UPDATE folder SET date_changed = now() 
   WHERE identifier = NEW.parent_folder;
END IF;

NEW.date_changed = now();
RETURN NEW;

END;
' LANGUAGE 'plpgsql' VOLATILE;

CREATE TRIGGER folder_update_trig 
 BEFORE INSERT OR UPDATE OR DELETE 
    ON folder
   FOR EACH ROW EXECUTE PROCEDURE date_changed_folder();


CREATE OR REPLACE FUNCTION date_changed_album() RETURNS "trigger" AS '
BEGIN

IF TG_OP = ''DELETE'' THEN
  UPDATE album SET date_changed = now() where identifier = OLD.parent_album;
  RETURN OLD;
END IF;

IF TG_OP = ''INSERT'' THEN
  NEW.date_of_creation = now();
  UPDATE album SET date_changed = now() 
   WHERE identifier = NEW.parent_album;
END IF;

NEW.date_changed = now();
RETURN NEW;

END;
' LANGUAGE 'plpgsql' VOLATILE;

CREATE TRIGGER album_update_trig 
 BEFORE INSERT OR UPDATE OR DELETE 
    ON album
   FOR EACH ROW EXECUTE PROCEDURE date_changed_album();


CREATE OR REPLACE FUNCTION date_changed_album_content() RETURNS "trigger" AS '
BEGIN

IF TG_OP = ''DELETE'' THEN
  UPDATE album SET date_changed = now() WHERE identifier = OLD.album;
  RETURN OLD;
END IF;

IF TG_OP = ''INSERT'' THEN
  UPDATE album SET date_changed = now() WHERE identifier = NEW.album;
END IF;

RETURN NEW;

END;
' LANGUAGE 'plpgsql' VOLATILE;

CREATE TRIGGER album_content_update_trig 
 BEFORE INSERT OR UPDATE OR DELETE 
    ON album_content
   FOR EACH ROW EXECUTE PROCEDURE date_changed_album_content();


-----  Eliminate Orphanage.

UPDATE photo SET folder =
   (SELECT identifier from folder f where f.users = photo.users
           AND f.caption = 'Trash')
  WHERE folder =
    (SELECT identifier from folder f2 where f2.users = photo.users
           AND f2.caption = 'Orphanage');

DELETE FROM folder where caption = 'Orphanage';

----- Shop is now gone.  Ensure preference is also gone.

DELETE FROM user_preferences where key = 'shop_enable';

-- version update II
update version set identifier=3, description='2.34 customer release', 
 major=2, minor=34, variation='C';

-- And one more trigger...

CREATE OR REPLACE FUNCTION date_changed_files() RETURNS "trigger" AS '
BEGIN

IF TG_OP = ''INSERT'' THEN
  NEW.created = now();
END IF;

RETURN NEW;

END;
' LANGUAGE 'plpgsql' VOLATILE;

CREATE TRIGGER date_changed_files_trig
 BEFORE INSERT OR UPDATE
    ON files
   FOR EACH ROW EXECUTE PROCEDURE date_changed_files();

--- Add some film formats.

update film_format set value = '35 mm' where value = '24x35 mm';
update film_format set value = '60x45 mm' where value = '45x60 mm';
update film_format set value = 'APS-C' where value = 'APS';

insert into film_format values (nextval('film_format_id_sequence'), '110-C');
insert into film_format values (nextval('film_format_id_sequence'), '126-C');
insert into film_format values (nextval('film_format_id_sequence'), '60x80 mm');
insert into film_format values (nextval('film_format_id_sequence'), '60x90 mm');
insert into film_format values (nextval('film_format_id_sequence'), '60x120 mm');
insert into film_format values (nextval('film_format_id_sequence'), '60x170 mm');
insert into film_format values (nextval('film_format_id_sequence'), '60x240 mm');

insert into film_format values (nextval('film_format_id_sequence'), '65x90 mm');
insert into film_format values (nextval('film_format_id_sequence'), '90x120 mm');
insert into film_format values (nextval('film_format_id_sequence'), '100x150 mm');
insert into film_format values (nextval('film_format_id_sequence'), '130x180 mm');
insert into film_format values (nextval('film_format_id_sequence'), '180x240 mm');
insert into film_format values (nextval('film_format_id_sequence'), '240x300 mm');
insert into film_format values (nextval('film_format_id_sequence'), '5x7 inch');
insert into film_format values (nextval('film_format_id_sequence'), '11x14 inch');

--- Delete the default 'No Caption' caption.

update photo set caption = '' where caption = 'No Caption';

