
--  Copyright (C) 2002, 2003, 2004, 2005, 2006 Balint Kis (balint@k-i-s.net)

--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.

--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.

--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

----------------------------------------------------------
-- v2.7 to v2.8 patch
----------------------------------------------------------

create sequence photo_format_id_sequence start 1;
create table photo_format (
  identifier            integer not null primary key,
  users                 integer references users(identifier),
  width                 varchar(100),
  height                varchar(100),
  date_of_creation      timestamp,
  date_of_last_edit     timestamp
);

create sequence currency_id_sequence start 1;
create table currency (
  identifier            integer not null primary key,
  numeric_code          numeric,
  alpha_code            varchar(5),
  name                  varchar(40),
  entity                text
);

create sequence photo_item_id_sequence start 1;
create table photo_item (
  identifier            integer not null primary key,
  photo                 integer not null references photo(identifier),
  format                integer not null references photo_format(identifier),
  price                 numeric,
  currency              integer references currency(identifier),
  comment               text,
  date_of_creation      timestamp,
  date_of_last_edit     timestamp
);

create sequence shop_transaction_status_id_sequence start 1;
create table shop_transaction_status (
  identifier            integer not null primary key,
  value                 varchar(15)
);

create sequence shop_transaction_id_sequence start 1;
create table shop_transaction (
  identifier            integer not null primary key,
  status                integer not null references shop_transaction_status(identifier),
  remark                text,
  date_of_creation      timestamp,
  date_of_last_edit     timestamp
);

create sequence shop_order_id_sequence start 1;
create table shop_order (
  identifier            integer not null primary key,
  client                integer references users(identifier),
  transaction           integer references shop_transaction,
  item                  integer not null references photo_item,
  pieces                numeric,
  date_of_creation      timestamp,
  date_of_last_edit     timestamp
);

alter table system_preferences add column new_user_shop_enable  char(1) check (new_user_shop_enable in ('t', 'f'));
alter table preferences add column shop_enable  char(1) check (shop_enable in ('t', 'f'));
alter table preferences add column popup_enable char(1) check (shop_enable in ('t', 'f'));
alter table preferences add column popup_delay  integer;

create function is_photo_for_sale(integer) returns bigint
as 'select count(photo_item.identifier) from photo, photo_item where photo.identifier=photo_item.photo and photo.identifier = $1' language 'sql';

-- Count photos per user for a specified access level.
-- A photo is accessible (visible), if its parent folder and the
-- photo itself has lower or equal access_rights to the specified one.
-- If the photo's visibility is restricted by its parent folder
-- we also take in consideration any album which might include the photo.

-- This is a temporary solution anyway. If we want to be accurate
-- we should make a recursive plpgsql script, counting all photos
-- of public directories, keeping the subselect for the albums.

drop function count_photos_by_user(integer, integer);
create function count_photos_by_user(integer, integer) returns bigint
as 'select count(photo.identifier)
      from photo, folder
     where photo.folder=folder.identifier
       and photo.users = $1
       and photo.access_rights <= $2
       and (folder.access_rights <= $2 or
(select album.access_rights
   from album, album_content
  where album_content.album=album.identifier
    and album_content.photo=photo.identifier
    and album.access_rights <= $2
  limit 1) <= $2) ' language 'sql';
